-- http://www.dasm.cz/clanek/zabbix-postgresql-a-table-partitioning

CREATE SCHEMA IF NOT EXISTS partitions AUTHORIZATION zabbix;

CREATE OR REPLACE FUNCTION zbx_history_partition() RETURNS TRIGGER AS $$
DECLARE
	parttable TEXT;
	startdate INT;
	enddate INT;
BEGIN
	-- Figure out the partition table name
	parttable := TG_TABLE_NAME || to_char(to_timestamp(NEW.clock), 'YYYYMM');
	-- Try inserting data to the partition
	EXECUTE 'INSERT INTO partitions.' || parttable || ' SELECT ($1).*' USING NEW;
	RETURN NULL;
	-- If the partition does not exist
	EXCEPTION WHEN undefined_table THEN
	-- Calculate the start and end dates for the respective day
	startdate := EXTRACT(epoch FROM date_trunc('month', to_timestamp(NEW.clock)));
	enddate := EXTRACT(epoch FROM date_trunc('month', to_timestamp(NEW.clock) + '1 month'::INTERVAL));
	-- Create new partition with indexes, constraints and all the stuff from parent table and add also the clock check
	EXECUTE 'CREATE TABLE partitions.' || parttable || ' (LIKE ' || TG_TABLE_NAME || ' INCLUDING ALL, CHECK (clock >= ' || startdate || ' AND clock < ' || enddate || ')) INHERITS (' || TG_TABLE_NAME || ')';
	-- Retry inserting to the newly created partition
	EXECUTE 'INSERT INTO partitions.' || parttable || ' SELECT ($1).*' USING NEW;
	RETURN NULL;
END
$$ LANGUAGE plpgsql;

CREATE TRIGGER tg_history_before_insert BEFORE INSERT ON history FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();
CREATE TRIGGER tg_history_uint_before_insert BEFORE INSERT ON history_uint FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();
CREATE TRIGGER tg_history_str_before_insert BEFORE INSERT ON history_str FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();
CREATE TRIGGER tg_history_text_before_insert BEFORE INSERT ON history_text FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();
CREATE TRIGGER tg_history_log_before_insert BEFORE INSERT ON history_log FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();
CREATE TRIGGER tg_trends_before_insert BEFORE INSERT ON trends FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();
CREATE TRIGGER tg_trends_uint_before_insert BEFORE INSERT ON trends_uint FOR EACH ROW EXECUTE PROCEDURE zbx_history_partition();


